function [nreps, means_allnu, stderrors_allnu, Except11, Except51, IExcept11, IExcept51, percents, total_percent_exceptions] = runallsims(nreps, sym, load)

%% Runs simP for all 12 cases of Tables 1 (sym = 0) and 2 (sym = 1); 
%% prepares output and stores in 'table1.mat' or 'table2.mat'

% First column of 'percents' is percentage of times P=I; 
% second is percentage of THOSE which were exceptions, with design non-uniform

% Except11 and Except51 analyze cases where the design is non-uniform and yet P = I
% IExcept11 and IExcept51 analyze cases where ~(P=I) and yet 
% the equations of Theorem 3(ii) do not both fail.


% Last four columns of the exception and Iexception outputs are 
% {diff0, diff1, degree, n}.

% Example: 
% [nreps, means_allnu, stderrors_allnu, Except11, Except51, IExcept11, IExcept51, percents, total_percent_exceptions] = runallsims(500, 1, 0);

if (load)
    if (sym == 1)
        load('table2.mat', 'nreps', 'means_allnu', 'stderrors_allnu', 'percents', 'total_percent_exceptions', 'Except11', 'Except51', 'IExcept11', 'IExcept51')
    elseif (sym == 0)
        load('table1.mat', 'nreps', 'means_allnu', 'stderrors_allnu', 'percents', 'total_percent_exceptions', 'Except11', 'Except51', 'IExcept11', 'IExcept51')
    end
else
    rng(999)

    disp('rows 1 and 2 of table')
    [meanMeasures_smallnu, seMeasures_smallnu, meanMeasures_bignu, seMeasures_bignu, numberI_both, exceptions11, exceptions51, Iexceptions11, Iexceptions51] = simP(11, 10, [.5, 1], 1, nreps, sym);
    means_allnu = [meanMeasures_smallnu; meanMeasures_bignu];
    stderrors_allnu = [seMeasures_smallnu; seMeasures_bignu];
    numberI_allnu = numberI_both;
    Except11 =  exceptions11;
    Except51 =  exceptions51;
    IExcept11 =  Iexceptions11;
    IExcept51 =  Iexceptions51;
  
    disp('rows 3 and 4 of table')
    [meanMeasures_smallnu, seMeasures_smallnu, meanMeasures_bignu, seMeasures_bignu, numberI_both, exceptions11, exceptions51, Iexceptions11, Iexceptions51] = simP(51, 10, [.5, 1], 1, nreps, sym);
    means_allnu = [means_allnu; meanMeasures_smallnu; meanMeasures_bignu];
    stderrors_allnu = [stderrors_allnu; seMeasures_smallnu; seMeasures_bignu];
    numberI_allnu = [numberI_allnu; numberI_both];
    Except11 = [Except11;exceptions11];
    Except51 = [Except51;exceptions51];
    IExcept11 = [IExcept11;Iexceptions11];
    IExcept51 = [IExcept51;Iexceptions51];

    disp('rows 5 and 6 of table')
    [meanMeasures_smallnu, seMeasures_smallnu, meanMeasures_bignu, seMeasures_bignu, numberI_both, exceptions11, exceptions51, Iexceptions11, Iexceptions51] = simP(11, 15, [.5, 1], 2, nreps, sym);
    means_allnu = [means_allnu; meanMeasures_smallnu; meanMeasures_bignu];
    stderrors_allnu = [stderrors_allnu; seMeasures_smallnu; seMeasures_bignu];
    numberI_allnu = [numberI_allnu; numberI_both];
    Except11 = [Except11;exceptions11];
    Except51 = [Except51;exceptions51];
    IExcept11 = [IExcept11;Iexceptions11];
    IExcept51 = [IExcept51;Iexceptions51];

    disp('rows 7 and 8 of table')
    [meanMeasures_smallnu, seMeasures_smallnu, meanMeasures_bignu, seMeasures_bignu, numberI_both, exceptions11, exceptions51, Iexceptions11, Iexceptions51] = simP(51, 15, [.5, 1], 2, nreps, sym);
    means_allnu = [means_allnu; meanMeasures_smallnu; meanMeasures_bignu];
    stderrors_allnu = [stderrors_allnu; seMeasures_smallnu; seMeasures_bignu];
    numberI_allnu = [numberI_allnu; numberI_both];
    Except11 = [Except11;exceptions11];
    Except51 = [Except51;exceptions51];
    IExcept11 = [IExcept11;Iexceptions11];
    IExcept51 = [IExcept51;Iexceptions51];


    disp('rows 9 and 10 of table')
    [meanMeasures_smallnu, seMeasures_smallnu, meanMeasures_bignu, seMeasures_bignu, numberI_both, exceptions11, exceptions51, Iexceptions11, Iexceptions51] = simP(11, 20, [.5, 1],3, nreps, sym);
    means_allnu = [means_allnu; meanMeasures_smallnu; meanMeasures_bignu];
    stderrors_allnu = [stderrors_allnu; seMeasures_smallnu; seMeasures_bignu];
    numberI_allnu = [numberI_allnu; numberI_both];
    Except11 = [Except11;exceptions11];
    Except51 = [Except51;exceptions51];
    IExcept11 = [IExcept11;Iexceptions11];
    IExcept51 = [IExcept51;Iexceptions51];

    disp('rows 11 and 12 of table')
    [meanMeasures_smallnu, seMeasures_smallnu, meanMeasures_bignu, seMeasures_bignu, numberI_both, exceptions11, exceptions51, Iexceptions11, Iexceptions51] = simP(51, 20, [.5, 1], 3, nreps, sym);
    means_allnu = [means_allnu; meanMeasures_smallnu; meanMeasures_bignu];
    stderrors_allnu = [stderrors_allnu; seMeasures_smallnu; seMeasures_bignu];
    numberI_allnu = [numberI_allnu; numberI_both];
    Except11 = [Except11;exceptions11];
    Except51 = [Except51;exceptions51];
    IExcept11 = [IExcept11;Iexceptions11];
    IExcept51 = [IExcept51;Iexceptions51];

    numberI = numberI_allnu;

    % Analyze exceptions

    if(size(Except11,1)>0)
        row1exceptions = Except11((Except11(:,14)==1).*(Except11(:,15)==.5)==1,:);
        row2exceptions = Except11((Except11(:,14)==1).*(Except11(:,15)==1)==1,:);
        row5exceptions = Except11((Except11(:,14)==2).*(Except11(:,15)==.5)==1,:);
        row6exceptions = Except11((Except11(:,14)==2).*(Except11(:,15)==1)==1,:);
        row9exceptions = Except11((Except11(:,14)==3).*(Except11(:,15)==.5)==1,:);
        row10exceptions = Except11((Except11(:,14)==3).*(Except11(:,15)==1)==1,:);
    else
        row1exceptions = [];
        row2exceptions = [];
        row5exceptions = [];
        row6exceptions = [];
        row9exceptions = [];
        row10exceptions = [];
    end

    if(size(Except51,1)>0)
        row3exceptions = Except51((Except51(:,14)==1).*(Except51(:,15)==.5)==1,:);
        row4exceptions = Except51((Except51(:,14)==1).*(Except51(:,15)==1)==1,:);
        row7exceptions = Except51((Except51(:,14)==3).*(Except51(:,15)==.5)==1,:);
        row8exceptions = Except51((Except51(:,14)==4).*(Except51(:,15)==1)==1,:);
        row11exceptions = Except51((Except51(:,14)==3).*(Except51(:,15)==.5)==1,:);
        row12exceptions = Except51((Except51(:,14)==3).*(Except51(:,15)==1)==1,:);
    else
        row3exceptions = [];
        row4exceptions = [];
        row7exceptions = [];
        row8exceptions = [];
        row11exceptions = [];
        row12exceptions = [];
    end

    number_exceptions = [size(row1exceptions,1); size(row2exceptions,1); ...
        size(row3exceptions,1); size(row4exceptions,1);...
        size(row5exceptions,1); size(row6exceptions,1);...
        size(row7exceptions,1); size(row8exceptions,1);...
        size(row9exceptions,1); size(row10exceptions,1);...
        size(row11exceptions,1); size(row12exceptions,1)];


    percents = [100*numberI/nreps, 100*number_exceptions./numberI];
    total_percent_exceptions = 100*sum(number_exceptions)/sum(numberI);
    if(isnan(total_percent_exceptions))
        total_percent_exceptions = 0;
    end
    percents(isnan(percents)) = 0;
    percents = round(percents);
    total_percent_exceptions = round(total_percent_exceptions);
    if (sym == 1)
        save('table2.mat', 'nreps', 'means_allnu', 'stderrors_allnu', 'percents', 'total_percent_exceptions', 'Except11', 'Except51', 'IExcept11', 'IExcept51')
    elseif (sym == 0)
        save('table1.mat', 'nreps', 'means_allnu', 'stderrors_allnu', 'percents', 'total_percent_exceptions', 'Except11', 'Except51', 'IExcept11', 'IExcept51')
    end

end

