function [meanMeasures_smallnu, seMeasures_smallnu, meanMeasures_bignu, seMeasures_bignu, numberI_both, exceptions11, exceptions51, Iexceptions11, Iexceptions51]  = ...
    simP(N, n, NU, degree, nreps, sym)
%% Computes the minimax P 'nreps' times; prepares data for Tables 1 and 2
%% N must be odd, for symmetry
%% Example: 
% [meanMeasures_smallnu, seMeasures_smallnu, meanMeasures_bignu, seMeasures_bignu, numberI_both, exceptions11, exceptions51, Iexceptions11, Iexceptions51] 
% = simP(51, 10, [.5, 1], 1, 10, 1)

% sym = 0 for multinomial sampling; type = 1 for more structure

% exceptions are cases where the design is non-uniform and yet P = I
% Iexceptions are cases where ~(P=I) and yet the equations of Theorem 3(ii) do not both fail.

% tolerance for the 'isidentity' function is set there

chi = linspace(-1,1,N)'; %design space  
if degree==1 
    F = [ones(N,1), chi]; % Model matrix (X=JF); linear regression
elseif degree==2
    F = [ones(N,1), chi, chi.^2]; % Model matrix (X=JF); quadratic regression
elseif degree==3
    F = [ones(N,1), chi, chi.^2, chi.^3]; % Model matrix (X=JF); cubic regression
end
p = size(F,2);
[Q,~] = qr(F, 'econ'); % Orthogonal basis for F


Measures_smallnu = zeros(nreps, 5);
Measures_bignu = Measures_smallnu;
countI_smallnu = 0;
countI_bignu = 0;
exceptions11 = [];
exceptions51 = [];
Iexceptions11 = [];
Iexceptions51 = [];

for rep = 1:nreps
    t = rand(1,N); % choose N probabilities
    t = t/sum(t);
    nxi = mnrnd(n,t); % frequencies summing to n of points from the N locations;
    if (sym == 1)
        xi = nxi/n;
        xi = xi + (1e-4)*rand(size(xi));
        [~, nxi] = roundXi(xi,n);
    end
    nxi = round(nxi);
    dplus = nxi(nxi>0);
    cdf = [0, cumsum(nxi)];
    J = zeros(n,N); % incidence matrix of design
    for j = 1:N
        if cdf(j+1) > cdf(j)
            J((cdf(j)+1):cdf(j+1),j) = ones(cdf(j+1) - cdf(j),1);
        end
    end

    I = eye(n);

    nu = NU(1);
    if (var(dplus)==0)
        P = I;
    else
        L = minroot(n, J, Q, nu, p); % Could use 'minprecision' (see below) instead
        P = L'*L;
        if(isidentity(P))
            P = I;
        end
    end
    P0 = Popt(J);
    if (isidentity(P))
        countI_smallnu = countI_smallnu + 1;
    end
    [lossP, varP, biasP] = Loss(P, J, Q, nu, p);
    [~, varP0, biasP0] = Loss(P0, J, Q, nu, p);
    [lossI, varI, biasI] = Loss(I, J, Q, nu, p);
    Plosses_smallnu = [lossP, varP, biasP];
    Ilosses_smallnu = [lossI, varI, biasI];
    if ((isidentity(P))&&(var(dplus)>0))
        diff0 = varP0 - varI;
        diff1 = biasI - biasP0;
        if N==11
            exceptions11 = [exceptions11; nxi, diff0, diff1, degree, nu];
        elseif N==51
            exceptions51 = [exceptions51; nxi, diff0, diff1, degree, nu];
        end
    end
    if (~(isidentity(P))&&~(isidentity(P0)))
        diff0 = varP0 - varI; %should be positive
        diff1 = 1e16*(biasI - biasP0);  %should be positive
        indi = (diff0 > 0 && diff1 > 0);
        if (indi==0)
            if N==11
                Iexceptions11 = [Iexceptions11; nxi, diff0, diff1, degree, nu];
            elseif N==51
                Iexceptions51 = [Iexceptions51; nxi, diff0, diff1, degree, nu];
            end
        end
    end

    nu = NU(2);
    if (var(dplus)==0)
        P = I;
    else
        L = minroot(n, J, Q, nu, p); 
        P = L'*L;
        if(isidentity(P))
            P = I;
        end
    end
    P0 = Popt(J);
    if (isidentity(P))
        countI_bignu = countI_bignu + 1;
    end
    [lossP, varP, biasP] = Loss(P, J, Q, nu, p);
    [~, varP0, biasP0] = Loss(P0, J, Q, nu, p);
    [lossI, varI, biasI] = Loss(I, J, Q, nu, p);
    Plosses_bignu = [lossP, varP, biasP];
    Ilosses_bignu = [lossI, varI, biasI];
    if ((isidentity(P))&&(var(dplus)>0))
        diff0 = varP0 - varI;
        diff1 = biasI - biasP0;
        if N==11
            exceptions11 = [exceptions11; nxi, diff0, diff1, degree, nu];
        elseif N==51
            exceptions51 = [exceptions51; nxi, diff0, diff1, degree, nu];
        end
    end
    if (~(isidentity(P))&&~(isidentity(P0)))
        diff0 = varP0 - varI; %should be positive
        diff1 = 1e16*(biasI - biasP0);  %should be positive
        indi = (diff0 > 0 && diff1 > 0);
        if (indi==0)
            if N==11
                Iexceptions11 = [Iexceptions11; nxi, diff0, diff1, degree, nu];
            elseif N==51
                Iexceptions51 = [Iexceptions51; nxi, diff0, diff1, degree, nu];
            end
        end
    end

    % Percent weighted (by \nu) improvement, relative to P = I (i.e. OLS):
    lossDecreaseOverOLS_smallnu  = 100*(Ilosses_smallnu(1) - Plosses_smallnu(1))/Ilosses_smallnu(1);
    lossDecreaseOverOLS_bignu  = 100*(Ilosses_bignu(1) - Plosses_bignu(1))/Ilosses_bignu(1);


    % Percent increase in variance, using this P, relative to P=I (i.e. OLS):
    varIncreaseOverOLS_smallnu = 100*(Plosses_smallnu(2) - Ilosses_smallnu(2))/Ilosses_smallnu(2);
    varIncreaseOverOLS_bignu = 100*(Plosses_bignu(2) - Ilosses_bignu(2))/Ilosses_bignu(2);

    % Percent decrease in bias, using this P, relative to P=I (i.e. OLS):
    biasDecreaseOverOLS_smallnu = 100*(Ilosses_smallnu(3) - Plosses_smallnu(3))/Plosses_smallnu(3);
    biasDecreaseOverOLS_bignu = 100*(Ilosses_bignu(3) - Plosses_bignu(3))/Plosses_bignu(3);

    Measures_smallnu(rep,:) = [Ilosses_smallnu(1), Plosses_smallnu(1), lossDecreaseOverOLS_smallnu, ...
        varIncreaseOverOLS_smallnu, biasDecreaseOverOLS_smallnu];
    Measures_bignu(rep,:) = [Ilosses_bignu(1), Plosses_bignu(1), lossDecreaseOverOLS_bignu, ...
        varIncreaseOverOLS_bignu, biasDecreaseOverOLS_bignu];
end
meanMeasures_smallnu = round(mean(Measures_smallnu, 1),2);
seMeasures_smallnu = round(std(Measures_smallnu, 1)/sqrt(nreps),2);
meanMeasures_bignu = round(mean(Measures_bignu, 1),2);
seMeasures_bignu = round(std(Measures_bignu, 1)/sqrt(nreps),2);
numberI_smallnu = countI_smallnu;
numberI_bignu = countI_bignu;
numberI_both = [numberI_smallnu; numberI_bignu];
 
%% ============================= subfunction "Popt" =============================
% Computes P0
function P = Popt(J)
[n,N] = size(J);
D = J'*J;
nvec = diag(D);
dplus = zeros(N,1);
for i = 1:N
    if nvec(i) > 0
        dplus(i) = 1/nvec(i);
    end
end
Dplus = diag(dplus); % not the same notation as in the article
alpha = n/sum(dplus);
P = alpha*J*Dplus*Dplus*J';

%% ============================= subfunction "minroot" =============================
% Minimizes the loss over triangular square roots of the precision matrix P
function L = minroot(n, J, Q, nu, p) % l0 = vecs L, lower triangular
func = @(l)lossroot(l, n, J, Q, nu, p);
nonlcon = @(l) mycon(l,n);
options = optimset('Display', 'none', 'algorithm', 'interior-point');
v = vecs(eye(n),n);
[l0, ~, ~] = fmincon(func, v,[],[],[],[],[],[], nonlcon, options); 
L = unvec(l0,n);

%% ============================= subfunction "mycon" =============================
function [c,ceq] = mycon(l,n)
c = [];
ceq = l'*l - n; 

%% ============================= subfunction "lossroot" =============================
%loss as function of root of P = L'*L; l = vecs L
function loss = lossroot(l, n, J, Q, nu, p) % l is n(n+1)/2 by 1
L = unvec(l,n);
% L is lower triangular, vecs(L) = l
[loss, ~, ~] = Loss(L'*L, J, Q, nu, p);

%% ============================= subfunction "loss" =============================
function [loss, varonly, biasonly] = Loss(P, J, Q, nu, p)
U = J'*P*J;
V = J'*P*P*J;
U0 = Q'*U*Q;
U1 = Q'*U*U*Q;
V0 = Q'*V*Q;
U0inv = U0\eye(p);
varonly = trace(U0inv*V0*U0inv');
biasonly = max(eig(U0inv*U1*U0inv'));
loss = (1-nu)*varonly + nu*biasonly;

%% ============================= subfunction "roundXi" =============================
function [xi, nxi] = roundXi(xi,n)
% Takes an N-point design and rounds it so give integer frequencies summing to n
N = length(xi); % N must be odd
xi = xi/sum(xi);
xi = (xi + flip(xi))/2;
nxi = round(n*xi);
if(sum(nxi) < n)
    toofew =  ceil(n*xi) - n*xi;
    [~,I] = sort(toofew, 'descend');
    indices = 1:(n - sum(nxi));
    nxi(I(indices)) = nxi(I(indices)) + 1;
elseif(sum(nxi) > n)
    toomany =  n*xi - floor(n*xi);
    [~,I] = sort(toomany, 'descend');
    indices = 1:(sum(nxi) - n);
    nxi(I(indices)) = nxi(I(indices)) - 1;
end
I = 1:N;
I1 = I(nxi > (nxi+flip(nxi))/2);
nxi(I1) = nxi(I1) - 1;
nxi((N+1)/2) = nxi((N+1)/2) + length(I1);
xi = nxi/n;

%% ============================= subfunction "unvec" =============================
function L = unvec(l,n)
L = zeros(n,n);
for j = 1:n
    jj = (j-1)*(n-(j-2)/2);
    L(:,j) = [zeros(j-1,1); l((jj+1):(jj + n-j+1))];
end
% L is lower triangular, vecs(L) = l

%% ============================= subfunction "vecs" =============================
%Forms a vector from the lower triangle of a matrix
function vec = vecs(mat, p)
vec = [];
for j = 1:p
	vec = [vec; mat(j:p,j)];
end
 

%% ============================= subfunction "isidentity" =============================
% tests that a matrix is the identity
function out = isidentity(mat)
tolerance = 1e-7;
out = (max(max(abs(mat - eye(size(mat))))) < tolerance);


