function  [P0, P, dplus, diff0, diff1, tol] = testexceptions
%% Tests specific designs judged to be 'exceptions' or 'Iexceptions'
%% Example:  [P0, P, dplus, diff0, diff1, tol] = testexceptions

 load('table2.mat', 'nreps', 'means_allnu', 'stderrors_allnu', 'percents',...
     'total_percent_exceptions', 'Except11', 'Except51', 'IExcept11', 'IExcept51') % symmetrized design
 %load('table1.mat', 'nreps', 'means_allnu', 'stderrors_allnu', 'percents',...
 %    'total_percent_exceptions', 'Except11', 'Except51', 'IExcept11', 'IExcept51')  % non-symmetrized design

chosenrow = 5;
mat = Except51;
size(mat)
nxi = mat(chosenrow,1:51); % next 4 items are 'diff0, diff1, degree, nu'. diff1 has been multiplied by 1e16
N = length(nxi);
n = sum(nxi);
degree = mat(chosenrow,N+3);
nu =  mat(chosenrow,N+4);
dplus = nxi(nxi>0);
I = eye(n);

chi = linspace(-1,1,N)'; %design space
if degree==1
    F = [ones(N,1), chi]; % Model matrix (X=JF); linear regression
elseif degree==2
    F = [ones(N,1), chi, chi.^2]; % Model matrix (X=JF); quadratic regression
elseif degree==3
    F = [ones(N,1), chi, chi.^2, chi.^3]; % Model matrix (X=JF); cubic regression
end
p = size(F,2);
[Q,~] = qr(F, 'econ'); % Orthogonal basis for F


cdf = [0, cumsum(nxi)];
J = zeros(n,N); % incidence matrix of design
for j = 1:N
    if cdf(j+1) > cdf(j)
        J((cdf(j)+1):cdf(j+1),j) = ones(cdf(j+1) - cdf(j),1);
    end
end


if (var(dplus)==0)
    P = I;
else
    L = minroot(n, J, Q, nu, p); % Could use 'minprecision' (see below) instead
    P = L'*L;
end

P0 = Popt(J);

[~, varonlyI, biasonlyI] = Loss(eye(n), J, Q, nu, p);
[~, varonlyP0, biasonlyP0] = Loss(P0, J, Q, nu, p);

diff0 = varonlyP0 - varonlyI; %should be positive
diff1 = biasonlyI - biasonlyP0;  %should be positive
tol =  max(max(abs(P-eye(size(P)))));

%% ============================= subfunction "minroot" =============================
% Minimizes the loss over triangular square roots of the precision matrix P
function L = minroot(n, J, Q, nu, p) % l0 = vecs L, lower triangular
func = @(l)lossroot(l, n, J, Q, nu, p);
nonlcon = @(l) mycon(l,n);
options = optimset('Display', 'none', 'algorithm', 'interior-point');
v = vecs(eye(n),n);
[l0, ~, ~] = fmincon(func, v,[],[],[],[],[],[], nonlcon, options); 
L = unvec(l0,n);

%% ============================= subfunction "mycon" =============================
function [c,ceq] = mycon(l,n)
c = [];
ceq = l'*l - n; 

%% ============================= subfunction "lossroot" =============================
%loss as function of root of P = L'*L; l = vecs L
function loss = lossroot(l, n, J, Q, nu, p) % l is n(n+1)/2 by 1
L = unvec(l,n);
% L is lower triangular, vecs(L) = l
[loss, ~, ~] = Loss(L'*L, J, Q, nu, p);

%% ============================= subfunction "Popt" =============================
% Computes what I believe to be the optimal P
function P = Popt(J)
[n,N] = size(J);
D = J'*J;
nvec = diag(D);
dplus = zeros(N,1);
for i = 1:N
    if nvec(i) > 0
        dplus(i) = 1/nvec(i);
    end
end
Dplus = diag(dplus);
alpha = n/sum(dplus);
P = alpha*J*Dplus*Dplus*J';

%% ============================= subfunction "loss" =============================
function [loss, varonly, biasonly] = Loss(P, J, Q, nu, p)
U = J'*P*J;
V = J'*P*P*J;
U0 = Q'*U*Q;
U1 = Q'*U*U*Q;
V0 = Q'*V*Q;
U0inv = U0\eye(p);
varonly = trace(U0inv*V0*U0inv');
biasonly = max(eig(U0inv*U1*U0inv'));
loss = (1-nu)*varonly + nu*biasonly;

%% ============================= subfunction "vecs" =============================
%Forms a vector from the lower triangle of a matrix
function vec = vecs(mat, p)
vec = [];
for j = 1:p
	vec = [vec; mat(j:p,j)];
end

%% ============================= subfunction "unvec" =============================
function L = unvec(l,n)
L = zeros(n,n);
for j = 1:n
    jj = (j-1)*(n-(j-2)/2);
    L(:,j) = [zeros(j-1,1); l((jj+1):(jj + n-j+1))];
end
% L is lower triangular, vecs(L) = l
